-----------------------------------
--- --- Agent-based models of Geographical Systems - chapter 28

--[[  Exploring Urban Dynamics in Latin American Cities Using an Agent-Based Simulation Approach

-- Joana Barros, Birbeck College

This model focuses on a specific kind of urban growth that happens in Latin American cities, called
‘peripherisation’. This is characterized by the formation of low-income residential
areas in the peripheral ring of the city and a perpetuation of a dynamic core-periphery
spatial pattern. 

The process of peripherisation consists of the expansion of the city
borders through the formation of peripheral low-income settlements that are then incorporated
into the city by a long-term process of expansion in which some of the lowincome
areas are recontextualised within the urban system and occupied by higher
economic groups while new low-income settlements keep emerging on the periphery.

Agents are divided into three economic groups (breeds) in a proportion based on
the division of Latin American society by income. The main difference between
agents lies in their economic group. The simulation model is underlain by an economic logic, 
although it is not an economic model. It assumes that, despite the economic differences, 
all agents have the same locational preferences, which means that they all want to locate
close to the areas served by infrastructure, with nearby commerce, job opportunities
and so on. Since in Third World cities these facilities are found mostly close
to the high-income residential areas, agents look for a place close to a highincome
group residential area. The behaviours of the three income groups are
differentiated by the restrictions imposed on their economic power. Thus, the
high-income group is able to locate in any place of its preference. 
The medium-income group can locate everywhere except where the high-income group 
is already located; and, in turn the low-income group (in dark-grey) can locate only 
in otherwise vacant space. Since some agents can occupy other agents’ cells, this means 
that the latter are ‘evicted’ and must find other places to settle. 
]]--

-- Model parameters

S_EMPTY  = 0   -- empty state
S_POOR   = 1   -- poor income family
S_MIDDLE = 2   -- middle income family
S_RICH   = 3   -- high-income family

P_POOR = 0.60    -- proportion of poor families
P_MIDDLE = 0.30  -- proportion of middle-income families
P_RICH  = 0.10   -- proportion of high-income families

-- creates a map observer to see the evolution of the agents in a cell space

function createMap (cellspace)

-- a map has a legend and an observer
	leg = Legend {
		grouping = "uniquevalue",
		colorBar = {
			{value = S_EMPTY, color = "white"},
			{value = S_POOR, color = "blue"},
			{value = S_MIDDLE,  color = "yellow"},
			{value = S_RICH,  color = "red"}
		}
	}
	map = Observer {
		type = "map",
		subject     = cellspace,
		attributes  = {"state"},
		legends = {leg}
	}
end

---- create a cellular space where the agents interact

function createCellSpace(model)

	model.cellspace = CellularSpace {
				xdim = model.cellSpaceDim,
				ydim = model.cellSpaceDim
	}
	model.cellspace:createNeighborhood{
		strategy = "vonneumann",
		self = false
	}
    -- initially, all the cells are empty

	forEachCell(model.cellspace, function(cell)
			cell.state = S_EMPTY
	end)

	-- find the mid-point of the cell space
	mid = (model.cellSpaceDim - 1 )/2
	model.centralCell = model.cellspace:getCell(Coord{x=mid, y= mid})
end

-- finds a place for the agent considering the peripherization logic

function findPlace(agent)

	found = false
	repeat		
		-- find a random place in the neighboorhood of where the agent is now
		place  = agent:getCell():getNeighborhood():sample()

		-- move the agent to the new cell
		agent:move(place) 
		
		-- is there someone else in the cell? 
		occupants = place:getAgents()
		
		if #(occupants) == 1 then
			-- if there is no one else, then the agent occupies the cell
			-- and changes the status of the cell 
			place.state = agent.class
			-- terminates the location procedure for this agent
			found = true
		else
		    -- there is someone else in the cell 
		    -- finds if the other occupant can be evicted
		    -- if the agent is richer than the occupant
		    -- then find a new place for the occupant
			forEachAgent(place, function(occupant)
				if (occupant.class < agent.class) then
					place.state = agent.class
					findPlace(occupant)
					found = true
				end
			end)
		end
	until found   -- moves the agent randomly until the finds a place to stay
end

-- creates an agent which is a family trying to find a place to live
function createAgent (model)

	model.family = Agent {
		-- in TerraME, all agents have an initialization function
		-- an agent is initialized with a propability of being rich, middle-class or poor
		init = function(self)
			if math.random() < P_POOR then
				self.class = S_POOR
			elseif (math.random() < P_POOR + P_MIDDLE) then
				self.class = S_MIDDLE
			else 
				self.class = S_RICH
			end
		end,
	
	-- in TerraME, all agents have an execution function
		execute = function(self)
		    -- the agents enter the space in the central cell
			self:enter (model.centralCell)
			-- tries to find a place for the agent
			findPlace(self) 
		end
	}
end

-- create a society of agents 
function createSociety (model)
	model.society = Society {
		instance = model.family,
		quantity = model.societyDim
	}
end

-- create an environment to link the cell space and the society
function createEnvironment (model)
	model.env = Environment { model.cellspace, model.society }
	-- the placement strategy is not defined here since the agents
	-- all enter space in the central cell. So we do not need to
	-- place them when we create the environment
	model.env:createPlacement{strategy = "void", max = 1}				
end


-- run the model 
function run (model)
	forEachAgent(model.society, function(agent)
		-- execute the agent
		agent:execute()
		-- notify the cell space
		model.cellspace:notify()
	end)
end

-- model implementation step=by-step
function AgentModel(model)

    -- create a cellular space
	createCellSpace (model)

	-- create Agent
	createAgent (model)

	-- create Society
	createSociety (model)

    -- put the agents in the cell space
	createEnvironment (model)

	-- create a Map to show the agents' behaviour
	createMap (model.cellspace)
	model.cellspace:notify()

	return model
end


model = AgentModel {

	cellSpaceDim  =  51,
	societyDim    =  1800,
}

run(model)